@@ -5,10 +5,9 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-from copy import deepcopy
-
+from ansible import constants as C
 from ansible.parsing.dataloader import DataLoader
-from ansible.vars.clean import strip_internal_keys
+from ansible.vars.clean import module_response_deepcopy, strip_internal_keys
 
 _IGNORE = ('failed', 'skipped')
 _PRESERVE = ('attempts', 'changed', 'retries')
@@ -67,16 +66,17 @@ class TaskResult:
 
     def needs_debugger(self, globally_enabled=False):
         _debugger = self._task_fields.get('debugger')
+        _ignore_errors = C.TASK_DEBUGGER_IGNORE_ERRORS and self._task_fields.get('ignore_errors')
 
         ret = False
-        if globally_enabled and (self.is_failed() or self.is_unreachable()):
+        if globally_enabled and ((self.is_failed() and not _ignore_errors) or self.is_unreachable()):
             ret = True
 
         if _debugger in ('always',):
             ret = True
         elif _debugger in ('never',):
             ret = False
-        elif _debugger in ('on_failed',) and self.is_failed():
+        elif _debugger in ('on_failed',) and self.is_failed() and not _ignore_errors:
             ret = True
         elif _debugger in ('on_unreachable',) and self.is_unreachable():
             ret = True
@@ -129,7 +129,7 @@ class TaskResult:
 
             result._result = x
         elif self._result:
-            result._result = deepcopy(self._result)
+            result._result = module_response_deepcopy(self._result)
 
             # actualy remove
             for remove_key in ignore:
