@@ -1,46 +1,34 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# (c) 2013, Hiroaki Nakamura <hnakamur@gmail.com>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright: (c) 2013, Hiroaki Nakamura <hnakamur@gmail.com>
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
-
 DOCUMENTATION = '''
 ---
 module: hostname
 author:
-    - "Adrian Likins (@alikins)"
-    - "Hideki Saito (@saito-hideki)"
+    - Adrian Likins (@alikins)
+    - Hideki Saito (@saito-hideki)
 version_added: "1.4"
 short_description: Manage hostname
 requirements: [ hostname ]
 description:
     - Set system's hostname, supports most OSs/Distributions, including those using systemd.
     - Note, this module does *NOT* modify C(/etc/hosts). You need to modify it yourself using other modules like template or replace.
-    - Windows, HP-UX and AIX are not currently supported
+    - Windows, HP-UX and AIX are not currently supported.
 options:
     name:
-        required: true
         description:
             - Name of the host
+        required: true
 '''
 
 EXAMPLES = '''
@@ -48,13 +36,19 @@ EXAMPLES = '''
     name: web01
 '''
 
+import os
 import socket
-from distutils.version import LooseVersion
+import traceback
 
-# import module snippets
-from ansible.module_utils.basic import *
-from ansible.module_utils.facts import *
-from ansible.module_utils._text import to_bytes, to_native
+from ansible.module_utils.basic import (
+    AnsibleModule,
+    get_distribution,
+    get_distribution_version,
+    get_platform,
+    load_platform_subclass,
+)
+from ansible.module_utils.facts.system.service_mgr import ServiceMgrFactCollector
+from ansible.module_utils._text import to_native
 
 
 class UnimplementedStrategy(object):
@@ -92,6 +86,7 @@ class UnimplementedStrategy(object):
         self.module.fail_json(
             msg='hostname module cannot be used on platform %s' % msg_platform)
 
+
 class Hostname(object):
     """
     This is a generic Hostname manipulation class that is subclassed
@@ -110,9 +105,9 @@ class Hostname(object):
         return load_platform_subclass(Hostname, args, kwargs)
 
     def __init__(self, module):
-        self.module       = module
-        self.name         = module.params['name']
-        if self.platform == 'Linux' and Facts(module).is_systemd_managed():
+        self.module = module
+        self.name = module.params['name']
+        if self.platform == 'Linux' and ServiceMgrFactCollector.is_systemd_managed(module):
             self.strategy = SystemdStrategy(module)
         else:
             self.strategy = self.strategy_class(module)
@@ -132,6 +127,7 @@ class Hostname(object):
     def set_permanent_hostname(self, name):
         self.strategy.set_permanent_hostname(name)
 
+
 class GenericStrategy(object):
     """
     This is a generic Hostname manipulation strategy class.
@@ -157,30 +153,30 @@ class GenericStrategy(object):
         name = self.module.params['name']
         current_name = self.get_current_hostname()
         if current_name != name:
-            self.set_current_hostname(name)
+            if not self.module.check_mode:
+                self.set_current_hostname(name)
             self.changed = True
 
     def update_permanent_hostname(self):
         name = self.module.params['name']
         permanent_name = self.get_permanent_hostname()
         if permanent_name != name:
-            self.set_permanent_hostname(name)
+            if not self.module.check_mode:
+                self.set_permanent_hostname(name)
             self.changed = True
 
     def get_current_hostname(self):
         cmd = [self.hostname_cmd]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
         return to_native(out).strip()
 
     def set_current_hostname(self, name):
         cmd = [self.hostname_cmd, name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
     def get_permanent_hostname(self):
         return None
@@ -189,8 +185,6 @@ class GenericStrategy(object):
         pass
 
 
-# ===========================================
-
 class DebianStrategy(GenericStrategy):
     """
     This is a Debian family Hostname manipulation strategy class - it edits
@@ -203,20 +197,18 @@ class DebianStrategy(GenericStrategy):
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("")
-            except IOError:
-                err = get_exception()
+            except IOError as e:
                 self.module.fail_json(msg="failed to write file: %s" %
-                    str(err))
+                                          to_native(e), exception=traceback.format_exc())
         try:
             f = open(self.HOSTNAME_FILE)
             try:
                 return f.read().strip()
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to read hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
     def set_permanent_hostname(self, name):
         try:
@@ -225,12 +217,10 @@ class DebianStrategy(GenericStrategy):
                 f.write("%s\n" % name)
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to update hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
-# ===========================================
 
 class SLESStrategy(GenericStrategy):
     """
@@ -243,20 +233,18 @@ class SLESStrategy(GenericStrategy):
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("")
-            except IOError:
-                err = get_exception()
+            except IOError as e:
                 self.module.fail_json(msg="failed to write file: %s" %
-                    str(err))
+                                          to_native(e), exception=traceback.format_exc())
         try:
             f = open(self.HOSTNAME_FILE)
             try:
                 return f.read().strip()
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to read hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
     def set_permanent_hostname(self, name):
         try:
@@ -265,12 +253,10 @@ class SLESStrategy(GenericStrategy):
                 f.write("%s\n" % name)
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to update hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
-# ===========================================
 
 class RedHatStrategy(GenericStrategy):
     """
@@ -289,10 +275,9 @@ class RedHatStrategy(GenericStrategy):
                         return v.strip()
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to read hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
     def set_permanent_hostname(self, name):
         try:
@@ -315,12 +300,10 @@ class RedHatStrategy(GenericStrategy):
                 f.writelines(lines)
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to update hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
-# ===========================================
 
 class AlpineStrategy(GenericStrategy):
     """
@@ -339,20 +322,18 @@ class AlpineStrategy(GenericStrategy):
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("")
-            except IOError:
-                err = get_exception()
+            except IOError as e:
                 self.module.fail_json(msg="failed to write file: %s" %
-                    str(err))
+                                          to_native(e), exception=traceback.format_exc())
         try:
             f = open(self.HOSTNAME_FILE)
             try:
                 return f.read().strip()
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to read hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
     def set_permanent_hostname(self, name):
         try:
@@ -361,21 +342,17 @@ class AlpineStrategy(GenericStrategy):
                 f.write("%s\n" % name)
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to update hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
     def set_current_hostname(self, name):
         cmd = [self.hostname_cmd, '-F', self.HOSTNAME_FILE]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
 
-# ===========================================
-
 class SystemdStrategy(GenericStrategy):
     """
     This is a Systemd hostname manipulation strategy class - it uses
@@ -386,8 +363,7 @@ class SystemdStrategy(GenericStrategy):
         cmd = ['hostname']
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
         return to_native(out).strip()
 
     def set_current_hostname(self, name):
@@ -396,15 +372,13 @@ class SystemdStrategy(GenericStrategy):
         cmd = ['hostnamectl', '--transient', 'set-hostname', name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
     def get_permanent_hostname(self):
         cmd = ['hostnamectl', '--static', 'status']
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
         return to_native(out).strip()
 
     def set_permanent_hostname(self, name):
@@ -413,17 +387,13 @@ class SystemdStrategy(GenericStrategy):
         cmd = ['hostnamectl', '--pretty', 'set-hostname', name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
         cmd = ['hostnamectl', '--static', 'set-hostname', name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
 
-# ===========================================
-
 class OpenRCStrategy(GenericStrategy):
     """
     This is a Gentoo (OpenRC) Hostname manipulation strategy class - it edits
@@ -440,9 +410,9 @@ class OpenRCStrategy(GenericStrategy):
                     line = line.strip()
                     if line.startswith('hostname='):
                         return line[10:].strip('"')
-            except Exception:
-                err = get_exception()
-                self.module.fail_json(msg="failed to read hostname: %s" % str(err))
+            except Exception as e:
+                self.module.fail_json(msg="failed to read hostname: %s" %
+                                          to_native(e), exception=traceback.format_exc())
         finally:
             f.close()
 
@@ -462,13 +432,12 @@ class OpenRCStrategy(GenericStrategy):
 
                 f = open(self.HOSTNAME_FILE, 'w')
                 f.write('\n'.join(lines) + '\n')
-            except Exception:
-                err = get_exception()
-                self.module.fail_json(msg="failed to update hostname: %s" % str(err))
+            except Exception as e:
+                self.module.fail_json(msg="failed to update hostname: %s" %
+                                          to_native(e), exception=traceback.format_exc())
         finally:
             f.close()
 
-# ===========================================
 
 class OpenBSDStrategy(GenericStrategy):
     """
@@ -482,20 +451,18 @@ class OpenBSDStrategy(GenericStrategy):
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("")
-            except IOError:
-                err = get_exception()
+            except IOError as e:
                 self.module.fail_json(msg="failed to write file: %s" %
-                    str(err))
+                                          to_native(e), exception=traceback.format_exc())
         try:
             f = open(self.HOSTNAME_FILE)
             try:
                 return f.read().strip()
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to read hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
     def set_permanent_hostname(self, name):
         try:
@@ -504,12 +471,10 @@ class OpenBSDStrategy(GenericStrategy):
                 f.write("%s\n" % name)
             finally:
                 f.close()
-        except Exception:
-            err = get_exception()
+        except Exception as e:
             self.module.fail_json(msg="failed to update hostname: %s" %
-                str(err))
+                                      to_native(e), exception=traceback.format_exc())
 
-# ===========================================
 
 class SolarisStrategy(GenericStrategy):
     """
@@ -522,8 +487,7 @@ class SolarisStrategy(GenericStrategy):
         cmd = [self.hostname_cmd, cmd_option, name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
     def get_permanent_hostname(self):
         fmri = 'svc:/system/identity:node'
@@ -531,18 +495,15 @@ class SolarisStrategy(GenericStrategy):
         cmd = '/usr/sbin/svccfg -s %s listprop -o value %s' % (fmri, pattern)
         rc, out, err = self.module.run_command(cmd, use_unsafe_shell=True)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
         return to_native(out).strip()
 
     def set_permanent_hostname(self, name):
         cmd = [self.hostname_cmd, name]
         rc, out, err = self.module.run_command(cmd)
         if rc != 0:
-            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
-                (rc, out, err))
+            self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" % (rc, out, err))
 
-# ===========================================
 
 class FreeBSDStrategy(GenericStrategy):
     """
@@ -557,10 +518,9 @@ class FreeBSDStrategy(GenericStrategy):
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("hostname=temporarystub\n")
-            except IOError:
-                err = get_exception()
+            except IOError as e:
                 self.module.fail_json(msg="failed to write file: %s" %
-                    str(err))
+                                          to_native(e), exception=traceback.format_exc())
         try:
             try:
                 f = open(self.HOSTNAME_FILE, 'r')
@@ -568,9 +528,9 @@ class FreeBSDStrategy(GenericStrategy):
                     line = line.strip()
                     if line.startswith('hostname='):
                         return line[10:].strip('"')
-            except Exception:
-                err = get_exception()
-                self.module.fail_json(msg="failed to read hostname: %s" % str(err))
+            except Exception as e:
+                self.module.fail_json(msg="failed to read hostname: %s" %
+                                          to_native(e), exception=traceback.format_exc())
         finally:
             f.close()
 
@@ -590,167 +550,252 @@ class FreeBSDStrategy(GenericStrategy):
 
                 f = open(self.HOSTNAME_FILE, 'w')
                 f.write('\n'.join(lines) + '\n')
-            except Exception:
-                err = get_exception()
-                self.module.fail_json(msg="failed to update hostname: %s" % str(err))
+            except Exception as e:
+                self.module.fail_json(msg="failed to update hostname: %s" %
+                                          to_native(e), exception=traceback.format_exc())
         finally:
             f.close()
 
-# ===========================================
 
 class FedoraHostname(Hostname):
     platform = 'Linux'
     distribution = 'Fedora'
     strategy_class = SystemdStrategy
 
+
 class SLESHostname(Hostname):
     platform = 'Linux'
     distribution = 'Suse linux enterprise server '
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion("10") <= LooseVersion(distribution_version) <= LooseVersion("12"):
-        strategy_class = SLESStrategy
-    else:
+    try:
+        distribution_version = get_distribution_version()
+        # cast to float may raise ValueError on non SLES, we use float for a little more safety over int
+        if distribution_version and 10 <= float(distribution_version) <= 12:
+            strategy_class = SLESStrategy
+        else:
+            raise ValueError()
+    except ValueError:
         strategy_class = UnimplementedStrategy
 
+
 class OpenSUSEHostname(Hostname):
     platform = 'Linux'
     distribution = 'Opensuse '
     strategy_class = SystemdStrategy
 
+
 class ArchHostname(Hostname):
     platform = 'Linux'
     distribution = 'Arch'
     strategy_class = SystemdStrategy
 
+
 class RedHat5Hostname(Hostname):
     platform = 'Linux'
     distribution = 'Redhat'
     strategy_class = RedHatStrategy
 
+
+class RHELHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Red hat enterprise linux'
+    strategy_class = RedHatStrategy
+
+
 class RedHatServerHostname(Hostname):
     platform = 'Linux'
     distribution = 'Red hat enterprise linux server'
     strategy_class = RedHatStrategy
 
+
 class RedHatWorkstationHostname(Hostname):
     platform = 'Linux'
     distribution = 'Red hat enterprise linux workstation'
     strategy_class = RedHatStrategy
 
+
+class RedHatAtomicHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Red hat enterprise linux atomic host'
+    strategy_class = RedHatStrategy
+
+
 class CentOSHostname(Hostname):
     platform = 'Linux'
     distribution = 'Centos'
     strategy_class = RedHatStrategy
 
+
 class CentOSLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Centos linux'
     strategy_class = RedHatStrategy
 
+
+class CloudlinuxHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Cloudlinux'
+    strategy_class = RedHatStrategy
+
+
+class CloudlinuxServerHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Cloudlinux server'
+    strategy_class = RedHatStrategy
+
+
 class ScientificHostname(Hostname):
     platform = 'Linux'
     distribution = 'Scientific'
     strategy_class = RedHatStrategy
 
+
 class ScientificLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Scientific linux'
     strategy_class = RedHatStrategy
 
+
 class ScientificLinuxCERNHostname(Hostname):
     platform = 'Linux'
     distribution = 'Scientific linux cern slc'
     strategy_class = RedHatStrategy
 
+
 class OracleLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Oracle linux server'
     strategy_class = RedHatStrategy
 
+
+class VirtuozzoLinuxHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Virtuozzo linux'
+    strategy_class = RedHatStrategy
+
+
 class AmazonLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Amazon'
     strategy_class = RedHatStrategy
 
+
 class DebianHostname(Hostname):
     platform = 'Linux'
     distribution = 'Debian'
     strategy_class = DebianStrategy
 
+
 class KaliHostname(Hostname):
     platform = 'Linux'
     distribution = 'Kali'
     strategy_class = DebianStrategy
 
+
 class UbuntuHostname(Hostname):
     platform = 'Linux'
     distribution = 'Ubuntu'
     strategy_class = DebianStrategy
 
+
 class LinuxmintHostname(Hostname):
     platform = 'Linux'
     distribution = 'Linuxmint'
     strategy_class = DebianStrategy
 
+
 class LinaroHostname(Hostname):
     platform = 'Linux'
     distribution = 'Linaro'
     strategy_class = DebianStrategy
 
+
+class DevuanHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Devuan'
+    strategy_class = DebianStrategy
+
+
 class GentooHostname(Hostname):
     platform = 'Linux'
     distribution = 'Gentoo base system'
     strategy_class = OpenRCStrategy
 
+
 class ALTLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Altlinux'
     strategy_class = RedHatStrategy
 
+
 class AlpineLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Alpine'
     strategy_class = AlpineStrategy
 
+
 class OpenBSDHostname(Hostname):
     platform = 'OpenBSD'
     distribution = None
     strategy_class = OpenBSDStrategy
 
+
 class SolarisHostname(Hostname):
     platform = 'SunOS'
     distribution = None
     strategy_class = SolarisStrategy
 
+
 class FreeBSDHostname(Hostname):
     platform = 'FreeBSD'
     distribution = None
     strategy_class = FreeBSDStrategy
 
+
 class NetBSDHostname(Hostname):
     platform = 'NetBSD'
     distribution = None
     strategy_class = FreeBSDStrategy
 
 
-# ===========================================
+class NeonHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Neon'
+    strategy_class = DebianStrategy
+
 
 def main():
     module = AnsibleModule(
-        argument_spec = dict(
+        argument_spec=dict(
             name=dict(required=True)
-        )
+        ),
+        supports_check_mode=True,
     )
 
     hostname = Hostname(module)
     name = module.params['name']
+
+    current_hostname = hostname.get_current_hostname()
+    permanent_hostname = hostname.get_permanent_hostname()
+
     changed = hostname.update_current_and_permanent_hostname()
 
-    module.exit_json(changed=changed, name=name,
-                     ansible_facts=dict(ansible_hostname=name.split('.')[0],
-                                        ansible_nodename=name,
-                                        ansible_fqdn=socket.getfqdn(),
-                                        ansible_domain='.'.join(socket.getfqdn().split('.')[1:])))
+    if name != current_hostname:
+        name_before = current_hostname
+    elif name != permanent_hostname:
+        name_before = permanent_hostname
+
+    kw = dict(changed=changed, name=name,
+              ansible_facts=dict(ansible_hostname=name.split('.')[0],
+                                 ansible_nodename=name,
+                                 ansible_fqdn=socket.getfqdn(),
+                                 ansible_domain='.'.join(socket.getfqdn().split('.')[1:])))
+
+    if changed:
+        kw['diff'] = {'after': 'hostname = ' + name + '\n',
+                      'before': 'hostname = ' + name_before + '\n'}
+
+    module.exit_json(**kw)
+
 
 if __name__ == '__main__':
     main()
